#include "functions.hpp"

void swap1(int a, int b)
{
    int temp_a = a;
    a = b;
    b = temp_a;
}

void swap2(int& a, int& b)
{
    // correct code
    int temp_a = a;
    a = b;
    b = temp_a;

    // wrong way to do it
    /*
    int& temp_a = a;
    a = b;
    b = temp_a;
    */
}

void swap3(int* a, int* b)
{
    // correct code
    int temp_a = *a;
    *a = *b;
    *b = temp_a;

    // wrong way to do it
    /*
    int* temp_a = a;
    *a = *b;
    *b = *temp_a;*/
}


void my_allocArr1(int* data, int size)
{
    // wrong code. Gets a copy of the pointer,
    // memory location pointed by new is lost
    // when the function is out of scope
    data = new int[size] {};
}

void my_allocArr2(int** data, int size)
{
    // Correct code. Gets a pointer to a pointer type variable.
    // Accesses that pointer and assigns the location of new.
    *data = new int[size]{};
}

void my_allocArr3(int*& data, int size)
{
    // Correct code. Gets an alias to a pointer type variable.
    // Referencing the pointer through the variable data is the same as 
    // referencing the actual variable.
    data = new int[size]{};
}

int* my_allocArr4(int size)
{
    // Correct code. Allocates the memory with new and returns the pointer.
    // The caller should keep a copy of that pointer type.

    return new int[size]{};

    // Also a valid way to do it.
    /*
    int* tmp = new int[size] {};
    return tmp;
    */
}

void my_destroy(int* data)
{
    delete[] data;
}

int& atIndex(int* intArr, int index)
{
    //return intArr[index];
    return *(intArr + index);
}